@@ -13,7 +13,7 @@ distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
-]]--
+]] --
 
 local exports = {}
 
@@ -27,11 +27,13 @@ local function check_violation(N, task, domain)
   -- Check for DKIM_REJECT
   local sym_check = 'R_DKIM_REJECT'
 
-  if N == 'arc' then sym_check = 'ARC_REJECT' end
+  if N == 'arc' then
+    sym_check = 'ARC_REJECT'
+  end
   if task:has_symbol(sym_check) then
     local sym = task:get_symbol(sym_check)[1]
     logger.infox(task, 'skip signing for %s: violation %s found: %s',
-        domain, sym_check, sym.options)
+      domain, sym_check, sym.options)
     return false
   end
 
@@ -90,9 +92,8 @@ local function parse_dkim_http_headers(N, task, settings)
     local key = task:get_request_header(headers.key_header)
 
     if not (domain and selector and key) then
-
       logger.errx(task, 'missing required headers to sign email')
-      return false,{}
+      return false, {}
     end
 
     -- Now check if we need to check the existing auth
@@ -114,23 +115,23 @@ local function parse_dkim_http_headers(N, task, settings)
   end
 
   lua_util.debugm(N, task, 'no sign header %s', headers.sign_header)
-  return false,{}
+  return false, {}
 end
 
 local function prepare_dkim_signing(N, task, settings)
   local is_local, is_sign_networks, is_authed
 
   if settings.use_http_headers then
-    local res,tbl = parse_dkim_http_headers(N, task, settings)
+    local res, tbl = parse_dkim_http_headers(N, task, settings)
 
     if not res then
       if not settings.allow_headers_fallback then
-        return res,{}
+        return res, {}
       else
         lua_util.debugm(N, task, 'failed to read http headers, fallback to normal schema')
       end
     else
-      return res,tbl
+      return res, tbl
     end
   end
 
@@ -139,13 +140,13 @@ local function prepare_dkim_signing(N, task, settings)
     local ret = settings.sign_condition(task)
 
     if not ret then
-      return false,{}
+      return false, {}
     end
 
     if ret[1] then
-      return true,ret
+      return true, ret
     else
-      return true,{ret}
+      return true, { ret }
     end
   end
 
@@ -163,19 +164,19 @@ local function prepare_dkim_signing(N, task, settings)
     if metric_action == 'reject' or metric_action == 'drop' then
       -- No need to sign what we are already rejecting/dropping
       lua_util.debugm(N, task, 'task result is already %s, no need to sign', metric_action)
-      return false,{}
+      return false, {}
     end
 
     if metric_action == 'soft reject' then
       -- Same here, we are going to delay an email, signing is just a waste of time
       lua_util.debugm(N, task, 'task result is %s, skip signing', metric_action)
-      return false,{}
+      return false, {}
     end
 
     -- For spam actions, there is no clear distinction
     if metric_action ~= 'no action' and type(settings.skip_spam_sign) == 'boolean' and settings.skip_spam_sign then
       lua_util.debugm(N, task, 'task result is %s, no need to sign', metric_action)
-      return false,{}
+      return false, {}
     end
   end
 
@@ -191,7 +192,7 @@ local function prepare_dkim_signing(N, task, settings)
     lua_util.debugm(N, task, 'mail was sent to us')
   else
     lua_util.debugm(N, task, 'mail is ineligible for signing')
-    return false,{}
+    return false, {}
   end
 
   local efrom = task:get_from('smtp')
@@ -199,7 +200,7 @@ local function prepare_dkim_signing(N, task, settings)
   if #(((efrom or E)[1] or E).addr or '') == 0 then
     if not settings.allow_envfrom_empty then
       lua_util.debugm(N, task, 'empty envelope from not allowed')
-      return false,{}
+      return false, {}
     else
       empty_envelope = true
     end
@@ -208,7 +209,7 @@ local function prepare_dkim_signing(N, task, settings)
   local hfrom = task:get_from('mime')
   if not settings.allow_hdrfrom_multiple and (hfrom or E)[2] then
     lua_util.debugm(N, task, 'multiple header from not allowed')
-    return false,{}
+    return false, {}
   end
 
   local eto = task:get_recipients(0)
@@ -256,29 +257,29 @@ local function prepare_dkim_signing(N, task, settings)
     -- OpenDKIM style
     if is_skip_sign() then
       lua_util.debugm(N, task,
-          'skip signing: is_sign_network: %s, is_authed: %s, is_local: %s',
-          is_sign_networks, is_authed, is_local)
-      return false,{}
+        'skip signing: is_sign_network: %s, is_authed: %s, is_local: %s',
+        is_sign_networks, is_authed, is_local)
+      return false, {}
     end
 
     if not hfrom or not hfrom[1] or not hfrom[1].addr then
       lua_util.debugm(N, task,
-          'signing_table: cannot get data when no header from is presented')
-      return false,{}
+        'signing_table: cannot get data when no header from is presented')
+      return false, {}
     end
-    local sign_entry = settings.signing_table:get_key(hfrom[1].addr)
+    local sign_entry = settings.signing_table:get_key(hfrom[1].addr:lower())
 
     if sign_entry then
       -- Check opendkim style entries
       lua_util.debugm(N, task,
-          'signing_table: found entry for %s: %s', hfrom[1].addr, sign_entry)
+        'signing_table: found entry for %s: %s', hfrom[1].addr, sign_entry)
       if sign_entry == '%' then
         sign_entry = hdom
       end
 
       if settings.key_table then
-      -- Now search in key table
-      local key_entry = settings.key_table:get_key(sign_entry)
+        -- Now search in key table
+        local key_entry = settings.key_table:get_key(sign_entry)
 
         if key_entry then
           local parts = lua_util.str_split(key_entry, ':')
@@ -289,8 +290,8 @@ local function prepare_dkim_signing(N, task, settings)
 
             if not selector then
               logger.errx(task, 'no selector defined for sign_entry %s, key_entry %s',
-                  sign_entry, key_entry)
-              return false,{}
+                sign_entry, key_entry)
+              return false, {}
             end
 
             local res = {
@@ -303,14 +304,14 @@ local function prepare_dkim_signing(N, task, settings)
             if st:sub(1, 1) == '/' or st == './' or st == '..' then
               res.key = parts[2]:gsub('%%', hdom)
               lua_util.debugm(N, task, 'perform dkim signing for %s, selector=%s, domain=%s, key file=%s',
-                  hdom, selector, res.domain, res.key)
+                hdom, selector, res.domain, res.key)
             else
               res.rawkey = parts[2] -- No sanity check here
               lua_util.debugm(N, task, 'perform dkim signing for %s, selector=%s, domain=%s, raw key used',
-                  hdom, selector, res.domain)
+                hdom, selector, res.domain)
             end
 
-            return true,{res}
+            return true, { res }
           elseif #parts == 3 then
             -- domain, selector, key
             local selector = parts[2]
@@ -325,56 +326,56 @@ local function prepare_dkim_signing(N, task, settings)
             if st:sub(1, 1) == '/' or st == './' or st == '..' then
               res.key = parts[3]:gsub('%%', hdom)
               lua_util.debugm(N, task, 'perform dkim signing for %s, selector=%s, domain=%s, key file=%s',
-                  hdom, selector, res.domain, res.key)
+                hdom, selector, res.domain, res.key)
             else
               res.rawkey = parts[3] -- No sanity check here
               lua_util.debugm(N, task, 'perform dkim signing for %s, selector=%s, domain=%s, raw key used',
-                  hdom, selector, res.domain)
+                hdom, selector, res.domain)
             end
 
-            return true,{res}
+            return true, { res }
           else
             logger.errx(task, 'invalid key entry for sign entry %s: %s; when signing %s domain',
-                sign_entry, key_entry, hdom)
-            return false,{}
+              sign_entry, key_entry, hdom)
+            return false, {}
           end
         elseif settings.use_vault then
           -- Sign table is presented, the rest is covered by vault
           lua_util.debugm(N, task, 'check vault for %s, by sign entry %s, key entry is missing',
-              hdom, sign_entry)
+            hdom, sign_entry)
           return true, {
             domain = sign_entry,
             vault = true
           }
         else
           logger.errx(task, 'missing key entry for sign entry %s; when signing %s domain',
-              sign_entry, hdom)
-          return false,{}
+            sign_entry, hdom)
+          return false, {}
         end
       else
         logger.errx(task, 'cannot get key entry for signing entry %s, when signing %s domain',
-            sign_entry, hdom)
-        return false,{}
+          sign_entry, hdom)
+        return false, {}
       end
     else
       lua_util.debugm(N, task,
-          'signing_table: no entry for %s', hfrom[1].addr)
-      return false,{}
+        'signing_table: no entry for %s', hfrom[1].addr)
+      return false, {}
     end
   else
     if settings.use_domain_sign_networks and is_sign_networks then
       dkim_domain = get_dkim_domain('use_domain_sign_networks')
       lua_util.debugm(N, task,
-          'sign_networks: use domain(%s) for signature: %s',
-          settings.use_domain_sign_networks, dkim_domain)
+        'sign_networks: use domain(%s) for signature: %s',
+        settings.use_domain_sign_networks, dkim_domain)
     elseif settings.use_domain_sign_local and is_local then
       dkim_domain = get_dkim_domain('use_domain_sign_local')
       lua_util.debugm(N, task, 'local: use domain(%s) for signature: %s',
-          settings.use_domain_sign_local, dkim_domain)
+        settings.use_domain_sign_local, dkim_domain)
     elseif settings.use_domain_sign_inbound and not is_local and not auser then
       dkim_domain = get_dkim_domain('use_domain_sign_inbound')
       lua_util.debugm(N, task, 'inbound: use domain(%s) for signature: %s',
-          settings.use_domain_sign_inbound, dkim_domain)
+        settings.use_domain_sign_inbound, dkim_domain)
     elseif settings.use_domain_custom then
       if type(settings.use_domain_custom) == 'string' then
         -- Load custom function
@@ -385,10 +386,10 @@ local function prepare_dkim_signing(N, task, settings)
             settings.use_domain_custom = res_or_err
             dkim_domain = settings.use_domain_custom(task)
             lua_util.debugm(N, task, 'use custom domain for signing: %s',
-                dkim_domain)
+              dkim_domain)
           else
             logger.errx(task, 'cannot load dkim domain custom script: invalid type: %s, expected function',
-                type(res_or_err))
+              type(res_or_err))
             settings.use_domain_custom = nil
           end
         else
@@ -398,18 +399,18 @@ local function prepare_dkim_signing(N, task, settings)
       else
         dkim_domain = settings.use_domain_custom(task)
         lua_util.debugm(N, task, 'use custom domain for signing: %s',
-            dkim_domain)
+          dkim_domain)
       end
     else
       dkim_domain = get_dkim_domain('use_domain')
       lua_util.debugm(N, task, 'use domain(%s) for signature: %s',
-          settings.use_domain, dkim_domain)
+        settings.use_domain, dkim_domain)
     end
   end
 
   if not dkim_domain then
     lua_util.debugm(N, task, 'could not extract dkim domain')
-    return false,{}
+    return false, {}
   end
 
   if settings.use_esld then
@@ -435,7 +436,7 @@ local function prepare_dkim_signing(N, task, settings)
         lua_util.debugm(N, task, 'domain mismatch allowed for empty envelope: %1 != %2', hdom, edom)
       else
         lua_util.debugm(N, task, 'domain mismatch not allowed: %1 != %2', hdom, edom)
-        return false,{}
+        return false, {}
       end
     end
   end
@@ -443,14 +444,14 @@ local function prepare_dkim_signing(N, task, settings)
   if auser and not settings.allow_username_mismatch then
     if not udom then
       lua_util.debugm(N, task, 'couldnt find domain in username')
-      return false,{}
+      return false, {}
     end
     if settings.use_esld then
       udom = rspamd_util.get_tld(udom)
     end
     if udom ~= dkim_domain then
       lua_util.debugm(N, task, 'user domain mismatch')
-      return false,{}
+      return false, {}
     end
   end
 
@@ -459,28 +460,27 @@ local function prepare_dkim_signing(N, task, settings)
   if settings.use_vault then
     if settings.vault_domains then
       if settings.vault_domains:get_key(dkim_domain) then
-        return true, {
+        table.insert(p, {
           domain = dkim_domain,
           vault = true,
-        }
+        })
       else
         lua_util.debugm(N, task, 'domain %s is not designated for vault',
           dkim_domain)
-        return false,{}
       end
     else
       -- TODO: try every domain in the vault
-      return true, {
+      table.insert(p, {
         domain = dkim_domain,
         vault = true,
-      }
+      })
     end
   end
 
   if settings.domain[dkim_domain] then
     -- support old style selector/paths
     if settings.domain[dkim_domain].selector or
-       settings.domain[dkim_domain].path then
+        settings.domain[dkim_domain].path then
       local k = {}
       k.selector = settings.domain[dkim_domain].selector
       k.key = settings.domain[dkim_domain].path
@@ -500,7 +500,7 @@ local function prepare_dkim_signing(N, task, settings)
     if ret then
       table.insert(p, k)
       lua_util.debugm(N, task, 'using mempool selector %s with key %s',
-                      k.selector, k.key)
+        k.selector, k.key)
     end
   end
 
@@ -524,27 +524,27 @@ local function prepare_dkim_signing(N, task, settings)
 
   if #p == 0 and not settings.try_fallback then
     lua_util.debugm(N, task, 'dkim unconfigured and fallback disabled')
-    return false,{}
+    return false, {}
   end
 
   if not settings.use_redis then
     insert_or_update_prop(N, task, p, 'key',
-        'default path', settings.path)
+      'default path', settings.path)
   end
 
   insert_or_update_prop(N, task, p, 'selector',
-        'default selector', settings.selector)
+    'default selector', settings.selector)
 
   if settings.check_violation then
     if not check_violation(N, task, p.domain) then
-      return false,{}
+      return false, {}
     end
   end
 
   insert_or_update_prop(N, task, p, 'domain', 'dkim_domain',
     dkim_domain)
 
-  return true,p
+  return #p > 0 and true or false, p
 end
 
 exports.prepare_dkim_signing = prepare_dkim_signing
@@ -559,53 +559,53 @@ exports.sign_using_redis = function(N, task, settings, selectors, sign_func, err
     local function redis_key_cb(err, data)
       if err then
         err_func(string.format("cannot make request to load DKIM key for %s: %s",
-            rk, err))
+          rk, err))
       elseif type(data) ~= 'string' then
         lua_util.debugm(N, task, "missing DKIM key for %s", rk)
       else
         p.rawkey = data
         lua_util.debugm(N, task, 'found and parsed key for %s:%s in Redis',
-            p.domain, p.selector)
+          p.domain, p.selector)
         sign_func(task, p)
       end
     end
     local rret = lua_redis.redis_make_request(task,
-        settings.redis_params, -- connect params
-        rk, -- hash key
-        false, -- is write
-        redis_key_cb, --callback
-        'HGET', -- command
-        {settings.key_prefix, rk} -- arguments
+      settings.redis_params,      -- connect params
+      rk,                         -- hash key
+      false,                      -- is write
+      redis_key_cb,               --callback
+      'HGET',                     -- command
+      { settings.key_prefix, rk } -- arguments
     )
     if not rret then
       err_func(task,
-          string.format( "cannot make request to load DKIM key for %s", rk))
+        string.format("cannot make request to load DKIM key for %s", rk))
     end
   end
 
   for _, p in ipairs(selectors) do
     if settings.selector_prefix then
       logger.infox(task, "using selector prefix '%s' for domain '%s'",
-          settings.selector_prefix, p.domain);
+        settings.selector_prefix, p.domain);
       local function redis_selector_cb(err, data)
         if err or type(data) ~= 'string' then
           err_func(task, string.format("cannot make request to load DKIM selector for domain %s: %s",
-              p.domain, err))
+            p.domain, err))
         else
           try_redis_key(data, p)
         end
       end
       local rret = lua_redis.redis_make_request(task,
-          settings.redis_params, -- connect params
-          p.domain, -- hash key
-          false, -- is write
-          redis_selector_cb, --callback
-          'HGET', -- command
-          {settings.selector_prefix, p.domain} -- arguments
+        settings.redis_params,                 -- connect params
+        p.domain,                              -- hash key
+        false,                                 -- is write
+        redis_selector_cb,                     --callback
+        'HGET',                                -- command
+        { settings.selector_prefix, p.domain } -- arguments
       )
       if not rret then
         err_func(task, string.format("cannot make Redis request to load DKIM selector for domain %s",
-            p.domain))
+          p.domain))
       end
     else
       try_redis_key(p.selector, p)
@@ -613,48 +613,53 @@ exports.sign_using_redis = function(N, task, settings, selectors, sign_func, err
   end
 end
 
-exports.sign_using_vault = function(N, task, settings, selectors, sign_func, err_func)
+exports.sign_using_vault = function(N, task, settings, selector, sign_func, err_func)
   local http = require "rspamd_http"
   local ucl = require "ucl"
 
   local full_url = string.format('%s/v1/%s/%s',
-      settings.vault_url, settings.vault_path or 'dkim', selectors.domain)
+    settings.vault_url, settings.vault_path or 'dkim', selector.domain)
   local upstream_list = lua_util.http_upstreams_by_url(rspamd_config:get_mempool(), settings.vault_url)
 
   local function vault_callback(err, code, body, _)
     if code ~= 200 then
       err_func(task, string.format('cannot request data from the vault url: %s; %s (%s)',
-          full_url, err, body))
+        full_url, err, body))
     else
       local parser = ucl.parser()
-      local res,parser_err = parser:parse_string(body)
+      local res, parser_err = parser:parse_string(body)
       if not res then
         err_func(task, string.format('vault reply for %s (data=%s) cannot be parsed: %s',
-            full_url, body, parser_err))
+          full_url, body, parser_err))
       else
         local obj = parser:get_object()
 
         if not obj or not obj.data then
           err_func(task, string.format('vault reply for %s (data=%s) is invalid, no data',
-              full_url, body))
+            full_url, body))
         else
           local elts = obj.data.selectors or {}
+          local errs = {}
+          local nvalid = 0
 
           -- Filter selectors by time/sanity
           local function is_selector_valid(p)
             if not p.key or not p.selector then
+              table.insert(errs, { "missing key/selector", p })
               return false
             end
 
             if p.valid_start then
               -- Check start time
               if rspamd_util.get_time() < tonumber(p.valid_start) then
+                table.insert(errs, { "start time is in the future", p })
                 return false
               end
             end
 
             if p.valid_end then
               if rspamd_util.get_time() >= tonumber(p.valid_end) then
+                table.insert(errs, { "end time is in the past", p })
                 return false
               end
             end
@@ -665,19 +670,28 @@ exports.sign_using_vault = function(N, task, settings, selectors, sign_func, err
             local dkim_sign_data = {
               rawkey = p.key,
               selector = p.selector,
-              domain = p.domain or selectors.domain,
+              domain = p.domain or selector.domain,
               alg = p.alg,
             }
             lua_util.debugm(N, task, 'found and parsed key for %s:%s in Vault',
-                dkim_sign_data.domain, dkim_sign_data.selector)
+              dkim_sign_data.domain, dkim_sign_data.selector)
+            nvalid = nvalid + 1
             sign_func(task, dkim_sign_data)
           end, fun.filter(is_selector_valid, elts))
+          for _, e in ipairs(errs) do
+            lua_util.debugm(N, task, 'error found during processing Vault selectors: %s:%s',
+              e[1], e[2])
+          end
+
+          if nvalid == 0 then
+            lua_util.debugm(N, task, 'no valid selectors have been returned from the Vault, skip signing')
+          end
         end
       end
     end
   end
 
-  local ret = http.request{
+  local ret = http.request {
     task = task,
     url = full_url,
     callback = vault_callback,
@@ -692,7 +706,7 @@ exports.sign_using_vault = function(N, task, settings, selectors, sign_func, err
 
   if not ret then
     err_func(task, string.format("cannot make HTTP request to load DKIM data domain %s",
-        selectors.domain))
+      selector.domain))
   end
 end
 
@@ -712,20 +726,19 @@ exports.process_signing_settings = function(N, settings, opts)
   local lua_maps = require "lua_maps"
   -- Used to convert plain options to the maps
   local maps_opts = {
-    sign_networks = {'radix', 'DKIM signing networks'},
-    path_map = {'map', 'Paths to DKIM signing keys'},
-    selector_map = {'map', 'DKIM selectors'},
-    signing_table = {'glob', 'DKIM signing table'},
-    key_table = {'glob', 'DKIM keys table'},
-    vault_domains = {'glob', 'DKIM signing domains in vault'},
-    whitelisted_signers_map = {'set', 'ARC trusted signers domains'}
+    sign_networks = { 'radix', 'DKIM signing networks' },
+    path_map = { 'map', 'Paths to DKIM signing keys' },
+    selector_map = { 'map', 'DKIM selectors' },
+    signing_table = { 'glob', 'DKIM signing table' },
+    key_table = { 'glob', 'DKIM keys table' },
+    vault_domains = { 'glob', 'DKIM signing domains in vault' }
   }
-  for k,v in pairs(opts) do
+  for k, v in pairs(opts) do
     local maybe_map = maps_opts[k]
     if maybe_map then
       settings[k] = lua_maps.map_add_from_ucl(v, maybe_map[1], maybe_map[2])
     elseif k == 'sign_condition' then
-      local ret,f = lua_util.callback_from_string(v)
+      local ret, f = lua_util.callback_from_string(v)
       if ret then
         settings[k] = f
       else
