@@ -26,13 +26,21 @@ local N = "lua_redis"
 
 local db_schema = (ts.number / tostring + ts.string):is_optional():describe("Database number")
 local common_schema = {
-  timeout = (ts.number + ts.string / lutil.parse_time_interval):is_optional():describe("Connection timeout"),
+  timeout = (ts.number + ts.string / lutil.parse_time_interval):is_optional():describe("Connection timeout (seconds)"),
   db = db_schema,
   database = db_schema,
   dbname = db_schema,
   prefix = ts.string:is_optional():describe("Key prefix"),
   username = ts.string:is_optional():describe("Username"),
   password = ts.string:is_optional():describe("Password"),
+  -- TLS options
+  ssl = ts.boolean:is_optional():describe("Enable TLS to Redis"),
+  no_ssl_verify = ts.boolean:is_optional():describe("Disable TLS certificate verification"),
+  ssl_ca = ts.string:is_optional():describe("CA certificate file"),
+  ssl_ca_dir = ts.string:is_optional():describe("CA certificates directory"),
+  ssl_cert = ts.string:is_optional():describe("Client certificate file (PEM)"),
+  ssl_key = ts.string:is_optional():describe("Client private key file (PEM)"),
+  sni = ts.string:is_optional():describe("SNI server name override"),
   expand_keys = ts.boolean:is_optional():describe("Expand keys"),
   sentinels = (ts.string + ts.array_of(ts.string)):is_optional():describe("Sentinel servers"),
   sentinel_watch_time = (ts.number + ts.string / lutil.parse_time_interval):is_optional():describe("Sentinel watch time"),
@@ -40,6 +48,7 @@ local common_schema = {
   sentinel_master_maxerrors = (ts.number + ts.string / tonumber):is_optional():describe("Sentinel master max errors"),
   sentinel_username = ts.string:is_optional():describe("Sentinel username"),
   sentinel_password = ts.string:is_optional():describe("Sentinel password"),
+  redis_version = (ts.number + ts.string / tonumber):is_optional():describe("Redis server version (6 or 7)"),
 }
 
 local read_schema = lutil.table_merge({
@@ -357,6 +366,10 @@ local function process_redis_opts(options, redis_params)
     redis_params['prefix'] = options['prefix']
   end
 
+  if options['redis_version'] and not redis_params['redis_version'] then
+    redis_params['redis_version'] = tonumber(options['redis_version'])
+  end
+
   if type(options['expand_keys']) == 'boolean' then
     redis_params['expand_keys'] = options['expand_keys']
   else
@@ -379,6 +392,29 @@ local function process_redis_opts(options, redis_params)
     redis_params['password'] = options['password']
   end
 
+  -- TLS passthrough
+  if options['ssl'] ~= nil and redis_params['ssl'] == nil then
+    redis_params['ssl'] = options['ssl'] and true or false
+  end
+  if options['no_ssl_verify'] ~= nil and redis_params['no_ssl_verify'] == nil then
+    redis_params['no_ssl_verify'] = options['no_ssl_verify'] and true or false
+  end
+  if options['ssl_ca'] and not redis_params['ssl_ca'] then
+    redis_params['ssl_ca'] = options['ssl_ca']
+  end
+  if options['ssl_ca_dir'] and not redis_params['ssl_ca_dir'] then
+    redis_params['ssl_ca_dir'] = options['ssl_ca_dir']
+  end
+  if options['ssl_cert'] and not redis_params['ssl_cert'] then
+    redis_params['ssl_cert'] = options['ssl_cert']
+  end
+  if options['ssl_key'] and not redis_params['ssl_key'] then
+    redis_params['ssl_key'] = options['ssl_key']
+  end
+  if options['sni'] and not redis_params['sni'] then
+    redis_params['sni'] = options['sni']
+  end
+
   if not redis_params.sentinels and options.sentinels then
     redis_params.sentinels = options.sentinels
   end
@@ -1017,6 +1053,15 @@ local function rspamd_redis_make_request(task, redis_params, key, is_write,
     options['dbname'] = redis_params['db']
   end
 
+  -- TLS options
+  if redis_params.ssl ~= nil then options.ssl = redis_params.ssl end
+  if redis_params.no_ssl_verify ~= nil then options.no_ssl_verify = redis_params.no_ssl_verify end
+  if redis_params.ssl_ca then options.ssl_ca = redis_params.ssl_ca end
+  if redis_params.ssl_ca_dir then options.ssl_ca_dir = redis_params.ssl_ca_dir end
+  if redis_params.ssl_cert then options.ssl_cert = redis_params.ssl_cert end
+  if redis_params.ssl_key then options.ssl_key = redis_params.ssl_key end
+  if redis_params.sni then options.sni = redis_params.sni end
+
   lutil.debugm(N, task, 'perform request to redis server' ..
       ' (host=%s, timeout=%s): cmd: %s', ip_addr,
       options.timeout, options.cmd)
@@ -1111,6 +1156,15 @@ local function redis_make_request_taskless(ev_base, cfg, redis_params, key,
     options['dbname'] = redis_params['db']
   end
 
+  -- TLS options
+  if redis_params.ssl ~= nil then options.ssl = redis_params.ssl end
+  if redis_params.no_ssl_verify ~= nil then options.no_ssl_verify = redis_params.no_ssl_verify end
+  if redis_params.ssl_ca then options.ssl_ca = redis_params.ssl_ca end
+  if redis_params.ssl_ca_dir then options.ssl_ca_dir = redis_params.ssl_ca_dir end
+  if redis_params.ssl_cert then options.ssl_cert = redis_params.ssl_cert end
+  if redis_params.ssl_key then options.ssl_key = redis_params.ssl_key end
+  if redis_params.sni then options.sni = redis_params.sni end
+
   lutil.debugm(N, cfg, 'perform taskless request to redis server' ..
       ' (host=%s, timeout=%s): cmd: %s', options.host:tostring(true),
       options.timeout, options.cmd)
@@ -1124,9 +1178,9 @@ local function redis_make_request_taskless(ev_base, cfg, redis_params, key,
 end
 
 --[[[
--- @function lua_redis.redis_make_request_taskless(ev_base, redis_params, key, is_write, callback, command, args)
+-- @function lua_redis.redis_make_request_taskless(ev_base, cfg, redis_params, key, is_write, callback, command, args)
 -- Sends a request to Redis in context where `task` is not available for some specific use-cases
--- Identical to redis_make_request() except in that first parameter is an `event base` object
+-- Identical to redis_make_request() except in that first parameter is an `event base` object and the second one is the 'config' object
 --]]
 
 exports.rspamd_redis_make_request_taskless = redis_make_request_taskless
@@ -1202,15 +1256,13 @@ local function prepare_redis_call(script)
   return options
 end
 
-local function is_all_servers_ready(script)
+local function is_any_server_ready(script)
   for _, s in ipairs(script.servers_ready) do
-    if s == "unsent" or s == "tempfail" then
-      return false
+    if s == "done" then
+      return true
     end
   end
-
-  -- We assume that permanent errors are not recoverable, so we will just skip those servers
-  return true
+  return false
 end
 
 local function is_all_servers_failed(script)
@@ -1264,7 +1316,7 @@ local function load_script_task(script, task, is_write)
           script.sha = data -- We assume that sha is the same on all servers
           script.servers_ready[idx] = "done"
         end
-        if is_all_servers_ready(script) then
+        if is_any_server_ready(script) then
           script_set_loaded(script)
         elseif is_all_servers_failed(script) then
           script.pending_upload = false
@@ -1282,7 +1334,7 @@ local function load_script_task(script, task, is_write)
       end
     end
 
-    if is_all_servers_ready(script) then
+    if is_any_server_ready(script) then
       script_set_loaded(script)
     elseif is_all_servers_failed(script) then
       script.pending_upload = false
@@ -1309,7 +1361,6 @@ local function load_script_taskless(script, cfg, ev_base, is_write)
                 err, script.caller.short_src, script.caller.currentline)
             opt.upstream:fail()
             script.servers_ready[idx] = "failed"
-            return
           else
             -- Assume temporary error
             logger.infox(cfg, 'temporary error uploading script %s to %s: %s; registered from: %s:%s',
@@ -1317,7 +1368,6 @@ local function load_script_taskless(script, cfg, ev_base, is_write)
                 opt.upstream:get_addr():to_string(true),
                 err, script.caller.short_src, script.caller.currentline)
             script.servers_ready[idx] = "tempfail"
-            return
           end
         else
           opt.upstream:ok()
@@ -1330,7 +1380,7 @@ local function load_script_taskless(script, cfg, ev_base, is_write)
           script.servers_ready[idx] = "done"
         end
 
-        if is_all_servers_ready(script) then
+        if is_any_server_ready(script) then
           script_set_loaded(script)
         elseif is_all_servers_failed(script) then
           script.pending_upload = false
@@ -1348,7 +1398,7 @@ local function load_script_taskless(script, cfg, ev_base, is_write)
       end
     end
 
-    if is_all_servers_ready(script) then
+    if is_any_server_ready(script) then
       script_set_loaded(script)
     elseif is_all_servers_failed(script) then
       script.pending_upload = false
@@ -1477,6 +1527,10 @@ local function exec_redis_script(id, params, callback, keys, args)
             script.sha = nil
             script.loaded = nil
             script.pending_upload = true
+	    -- We must initialize all servers as we don't know here which one failed
+	    for i, _ in ipairs(script.servers_ready) do
+	      script.servers_ready[i] = "unsent"
+	    end
             -- Reload scripts if this has not been initiated yet
             if params.task then
               load_script_task(script, params.task)
@@ -1510,15 +1564,20 @@ local function exec_redis_script(id, params, callback, keys, args)
       end
     end
 
+    local redis_command = 'EVALSHA'
+    if not params.is_write and script.redis_params.redis_version and
+        script.redis_params.redis_version >= 7 then
+      redis_command = 'EVALSHA_RO'
+    end
     if params.task then
       if not rspamd_redis_make_request(params.task, script.redis_params,
-          params.key, params.is_write, redis_cb, 'EVALSHA', redis_args) then
+          params.key, params.is_write, redis_cb, redis_command, redis_args) then
         callback('Cannot make redis request', nil)
       end
     else
       if not redis_make_request_taskless(params.ev_base, rspamd_config,
           script.redis_params,
-          params.key, params.is_write, redis_cb, 'EVALSHA', redis_args) then
+          params.key, params.is_write, redis_cb, redis_command, redis_args) then
         callback('Cannot make redis request', nil)
       end
     end
@@ -1738,6 +1797,15 @@ exports.request = function(redis_params, attrs, req)
     opts.dbname = redis_params.db
   end
 
+  -- TLS options
+  if redis_params.ssl ~= nil then opts.ssl = redis_params.ssl end
+  if redis_params.no_ssl_verify ~= nil then opts.no_ssl_verify = redis_params.no_ssl_verify end
+  if redis_params.ssl_ca then opts.ssl_ca = redis_params.ssl_ca end
+  if redis_params.ssl_ca_dir then opts.ssl_ca_dir = redis_params.ssl_ca_dir end
+  if redis_params.ssl_cert then opts.ssl_cert = redis_params.ssl_cert end
+  if redis_params.ssl_key then opts.ssl_key = redis_params.ssl_key end
+  if redis_params.sni then opts.sni = redis_params.sni end
+
   if opts.callback then
     lutil.debugm(N, 'perform generic async request to redis server' ..
         ' (host=%s, timeout=%s): cmd: %s, arguments: %s', addr,
@@ -1843,6 +1911,15 @@ exports.connect = function(redis_params, attrs)
     opts.dbname = redis_params.db
   end
 
+  -- TLS options
+  if redis_params.ssl ~= nil then opts.ssl = redis_params.ssl end
+  if redis_params.no_ssl_verify ~= nil then opts.no_ssl_verify = redis_params.no_ssl_verify end
+  if redis_params.ssl_ca then opts.ssl_ca = redis_params.ssl_ca end
+  if redis_params.ssl_ca_dir then opts.ssl_ca_dir = redis_params.ssl_ca_dir end
+  if redis_params.ssl_cert then opts.ssl_cert = redis_params.ssl_cert end
+  if redis_params.ssl_key then opts.ssl_key = redis_params.ssl_key end
+  if redis_params.sni then opts.sni = redis_params.sni end
+
   if opts.callback then
     local ret, conn = rspamd_redis.connect(opts)
     if not ret then
