@@ -48,6 +48,8 @@ local function kaspersky_se_config(opts)
     scan_mime_parts = true,
     scan_text_mime = false,
     scan_image_mime = false,
+    keepalive = true,
+    auth_string = nil
   }
 
   default_conf = lua_util.override_defaults(default_conf, opts)
@@ -118,6 +120,15 @@ local function kaspersky_se_check(task, content, digest, rule, maybe_part)
       ['X-KAV-Timeout'] = tostring(rule.timeout * 1000),
     }
 
+    local ip = task:get_from_ip()
+    if ip and ip:is_valid() then
+      hdrs['X-KAV-HostIP'] = tostring(ip)
+    end
+
+    if rule.auth_string then
+      hdrs['Authorization'] = rule.auth_string
+    end
+
     if task:has_from() then
       hdrs['X-KAV-ObjectURL'] = string.format('[from:%s]', task:get_from()[1].addr)
     end
@@ -125,7 +136,7 @@ local function kaspersky_se_check(task, content, digest, rule, maybe_part)
     local req_body
 
     if rule.use_files then
-      local fname =  string.format('%s/%s.tmp',
+      local fname = string.format('%s/%s.tmp',
           rule.tmpdir, rspamd_util.random_hex(32))
       local message_fd = rspamd_util.create_file(fname)
 
@@ -158,6 +169,7 @@ local function kaspersky_se_check(task, content, digest, rule, maybe_part)
       body = req_body,
       headers = hdrs,
       timeout = rule.timeout,
+      keepalive = rule.keepalive,
     }
 
     local function kas_callback(http_err, code, body, headers)
@@ -183,12 +195,13 @@ local function kaspersky_se_check(task, content, digest, rule, maybe_part)
           lua_util.debugm(rule.name, task, '%s: retry IP: %s:%s',
               rule.log_prefix, addr, addr:get_port())
           request_data.url = url
+          request_data.upstream = upstream
 
           http.request(request_data)
         else
-          rspamd_logger.errx(task, '%s: failed to scan, maximum retransmits '..
+          rspamd_logger.errx(task, '%s: failed to scan, maximum retransmits ' ..
               'exceed', rule.log_prefix)
-          task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan and '..
+          task:insert_result(rule['symbol_fail'], 0.0, 'failed to scan and ' ..
               'retransmits exceed')
         end
       end
@@ -197,7 +210,9 @@ local function kaspersky_se_check(task, content, digest, rule, maybe_part)
         requery()
       else
         -- Parse the response
-        if upstream then upstream:ok() end
+        if upstream then
+          upstream:ok()
+        end
         if code ~= 200 then
           rspamd_logger.errx(task, 'invalid HTTP code: %s, body: %s, headers: %s', code, body, headers)
           task:insert_result(rule.symbol_fail, 1.0, 'Bad HTTP code: ' .. code)
@@ -241,7 +256,7 @@ local function kaspersky_se_check(task, content, digest, rule, maybe_part)
 
           if why == 'PASSWORD PROTECTED' then
             rspamd_logger.errx(task, '%s: File is encrypted', rule.log_prefix)
-            common.yield_result(task, rule, 'File is encrypted: '.. why,
+            common.yield_result(task, rule, 'File is encrypted: ' .. why,
                 0.0, 'encrypted', maybe_part)
             cached = 'ENCRYPTED'
           else
