@@ -20,19 +20,28 @@ local E = {}
 
 if confighelp then
   rspamd_config:add_example(nil, 'gpt',
-      "Performs postfiltering using GPT model",
-      [[
+    "Performs postfiltering using GPT model",
+    [[
   gpt {
   # Supported types: openai, ollama
   type = "openai";
   # Your key to access the API
   api_key = "xxx";
   # Model name
-  model = "gpt-4o-mini";
-  # Maximum tokens to generate
-  max_tokens = 1000;
-  # Temperature for sampling
-  temperature = 0.0;
+  model = "gpt-5-mini"; # or parallel model requests [ "gpt-5-mini", "gpt-4o-mini" ];
+  # Per-model parameters
+  model_parameters = {
+    "gpt-5-mini" = {
+      max_completion_tokens = 1000,
+    },
+    "gpt-5-nano" = {
+      max_completion_tokens = 1000,
+    },
+    "gpt-4o-mini" = {
+      max_tokens = 1000,
+      temperature = 0.0,
+    }
+  };
   # Timeout for requests
   timeout = 10s;
   # Prompt for the model (use default if not set)
@@ -62,9 +71,10 @@ local lua_util = require "lua_util"
 local rspamd_http = require "rspamd_http"
 local rspamd_logger = require "rspamd_logger"
 local lua_mime = require "lua_mime"
+local llm_common = require "llm_common"
 local lua_redis = require "lua_redis"
 local ucl = require "ucl"
-local fun = require "fun"
+-- local fun = require "fun" -- no longer needed after llm_common usage
 local lua_cache = require "lua_cache"
 
 -- Exclude checks if one of those is found
@@ -107,10 +117,25 @@ local categories_map = {}
 local settings = {
   type = 'openai',
   api_key = nil,
-  model = 'gpt-4o-mini',
-  max_tokens = 1000,
-  temperature = 0.0,
+  model = 'gpt-5-mini', -- or parallel model requests: [ 'gpt-5-mini', 'gpt-4o-mini' ],
+  model_parameters = {
+    ["gpt-5-mini"] = {
+      max_completion_tokens = 1000,
+    },
+    ["gpt-5-nano"] = {
+      max_completion_tokens = 1000,
+    },
+    ["gpt-4o-mini"] = {
+      max_tokens = 1000,
+      temperature = 0.0,
+    }
+  },
   timeout = 10,
+  -- Optional staged timeouts
+  connect_timeout = nil,
+  ssl_timeout = nil,
+  write_timeout = nil,
+  read_timeout = nil,
   prompt = nil,
   condition = nil,
   autolearn = false,
@@ -162,7 +187,7 @@ local function default_condition(task)
             end
           end
           lua_util.debugm(N, task, 'symbol %s has weight %s, but required %s', s,
-              sym.weight, required_weight)
+            sym.weight, required_weight)
         else
           return false, 'skip as "' .. s .. '" is found'
         end
@@ -182,7 +207,7 @@ local function default_condition(task)
             end
           end
           lua_util.debugm(N, task, 'symbol %s has weight %s, but required %s', s,
-              sym.weight, required_weight)
+            sym.weight, required_weight)
         end
       else
         return false, 'skip as "' .. s .. '" is not found'
@@ -190,29 +215,21 @@ local function default_condition(task)
     end
   end
 
-  -- Check if we have text at all
-  local sel_part = lua_mime.get_displayed_text_part(task)
-
+  -- Unified LLM input building (subject/from/urls/body one-line)
+  local model_cfg = settings.model_parameters[settings.model] or {}
+  local max_tokens = model_cfg.max_completion_tokens or model_cfg.max_tokens or 1000
+  local input_tbl, sel_part = llm_common.build_llm_input(task, { max_tokens = max_tokens })
   if not sel_part then
     return false, 'no text part found'
   end
-
-  -- Check limits and size sanity
-  local nwords = sel_part:get_words_count()
-
-  if nwords < 5 then
-    return false, 'less than 5 words'
-  end
-
-  if nwords > settings.max_tokens then
-    -- We need to truncate words (sometimes get_words_count returns a different number comparing to `get_words`)
-    local words = sel_part:get_words('norm')
-    nwords = #words
-    if nwords > settings.max_tokens then
-      return true, table.concat(words, ' ', 1, settings.max_tokens), sel_part
+  if not input_tbl then
+    local nwords = sel_part:get_words_count() or 0
+    if nwords < 5 then
+      return false, 'less than 5 words'
     end
+    return false, 'no content to send'
   end
-  return true, sel_part:get_content_oneline(), sel_part
+  return true, input_tbl, sel_part
 end
 
 local function maybe_extract_json(str)
@@ -253,6 +270,15 @@ local function maybe_extract_json(str)
   return nil
 end
 
+-- Helper function to remove <think>...</think> and trim leading newlines
+local function clean_gpt_response(text)
+  -- Remove <think>...</think> including multiline
+  text = text:gsub("<think>.-</think>", "")
+  -- Trim leading whitespace and newlines
+  text = text:gsub("^%s*\n*", "")
+  return text
+end
+
 local function default_openai_json_conversion(task, input)
   local parser = ucl.parser()
   local res, err = parser:parse_string(input)
@@ -301,7 +327,7 @@ local function default_openai_json_conversion(task, input)
       elseif reply.probability == "low" then
         spam_score = 0.1
       else
-        rspamd_logger.infox("cannot convert to spam probability: %s", reply.probability)
+        rspamd_logger.infox(task, "cannot convert to spam probability: %s", reply.probability)
       end
     end
 
@@ -324,7 +350,7 @@ local function clean_reply_line(line)
   return lua_util.str_trim(line):gsub("^%d%.%s+", "")
 end
 
--- Assume that we have 3 lines: probability, reason, additional symbols
+-- Assume that we have 3 lines: probability, reason, additional categories
 local function default_openai_plain_conversion(task, input)
   local parser = ucl.parser()
   local res, err = parser:parse_string(input)
@@ -345,10 +371,14 @@ local function default_openai_plain_conversion(task, input)
 
   local first_message = reply.choices[1].message.content
 
-  if not first_message then
+  if not first_message or first_message == "" then
     rspamd_logger.errx(task, 'no content in the first message')
     return
   end
+
+  -- Clean message
+  first_message = clean_gpt_response(first_message)
+
   local lines = lua_util.str_split(first_message, '\n')
   local first_line = clean_reply_line(lines[1])
   local spam_score = tonumber(first_line)
@@ -367,15 +397,6 @@ local function default_openai_plain_conversion(task, input)
   return
 end
 
--- Helper function to remove <think>...</think> and trim leading newlines
-local function clean_gpt_response(text)
-  -- Remove <think>...</think> including multiline
-  text = text:gsub("<think>.-</think>", "")
-  -- Trim leading whitespace and newlines
-  text = text:gsub("^%s*\n*", "")
-  return text
-end
-
 local function default_ollama_plain_conversion(task, input)
   local parser = ucl.parser()
   local res, err = parser:parse_string(input)
@@ -466,7 +487,7 @@ local function default_ollama_json_conversion(task, input)
       elseif reply.probability == "low" then
         spam_score = 0.1
       else
-        rspamd_logger.infox("cannot convert to spam probability: %s", reply.probability)
+        rspamd_logger.infox(task, "cannot convert to spam probability: %s", reply.probability)
       end
     end
 
@@ -474,7 +495,7 @@ local function default_ollama_json_conversion(task, input)
       rspamd_logger.infox(task, 'usage: %s tokens', reply.usage.total_tokens)
     end
 
-    return spam_score, reply.reason
+    return spam_score, reply.reason, {}
   end
 
   rspamd_logger.errx(task, 'cannot convert spam score: %s', first_message)
@@ -494,7 +515,7 @@ local function redis_cache_key(sel_part)
     env_digest = digest:hex():sub(1, 4)
   end
   return string.format('%s_%s', env_digest,
-      sel_part:get_mimepart():get_digest():sub(1, 24))
+    sel_part:get_mimepart():get_digest():sub(1, 24))
 end
 
 local function process_categories(task, categories)
@@ -531,8 +552,10 @@ local function insert_results(task, result, sel_part)
     end
   end
   if result.reason and settings.reason_header then
+    local v = lua_util.fold_header_with_encoding(task, settings.reason_header,
+      tostring(result.reason), { encode = 'auto' })
     lua_mime.modify_headers(task,
-        { add = { [settings.reason_header] = { value = tostring(result.reason), order = 1 } } })
+      { add = { [settings.reason_header] = { value = v, order = 1 } } })
   end
 
   if cache_context then
@@ -557,12 +580,12 @@ local function check_consensus_and_insert_results(task, results, sel_part)
         nspam = nspam + 1
         max_spam_prob = math.max(max_spam_prob, result.probability)
         lua_util.debugm(N, task, "model: %s; spam: %s; reason: '%s'",
-            result.model, result.probability, result.reason)
+          result.model, result.probability, result.reason)
       else
         nham = nham + 1
         max_ham_prob = math.min(max_ham_prob, result.probability)
         lua_util.debugm(N, task, "model: %s; ham: %s; reason: '%s'",
-            result.model, result.probability, result.reason)
+          result.model, result.probability, result.reason)
       end
 
       if result.reason then
@@ -576,41 +599,25 @@ local function check_consensus_and_insert_results(task, results, sel_part)
 
   if nspam > nham and max_spam_prob > 0.75 then
     insert_results(task, {
-      probability = max_spam_prob,
-      reason = reason.reason,
-      categories = reason.categories,
-    },
-        sel_part)
+        probability = max_spam_prob,
+        reason = reason.reason,
+        categories = reason.categories,
+      },
+      sel_part)
   elseif nham > nspam and max_ham_prob < 0.25 then
     insert_results(task, {
-      probability = max_ham_prob,
-      reason = reason.reason,
-      categories = reason.categories,
-    },
-        sel_part)
+        probability = max_ham_prob,
+        reason = reason.reason,
+        categories = reason.categories,
+      },
+      sel_part)
   else
     -- No consensus
     lua_util.debugm(N, task, "no consensus")
   end
-
 end
 
-local function get_meta_llm_content(task)
-  local url_content = "Url domains: no urls found"
-  if task:has_urls() then
-    local urls = lua_util.extract_specific_urls { task = task, limit = 5, esld_limit = 1 }
-    url_content = "Url domains: " .. table.concat(fun.totable(fun.map(function(u)
-      return u:get_tld() or ''
-    end, urls or {})), ', ')
-  end
-
-  local from_or_empty = ((task:get_from('mime') or E)[1] or E)
-  local from_content = string.format('From: %s <%s>', from_or_empty.name, from_or_empty.addr)
-  lua_util.debugm(N, task, "gpt urls: %s", url_content)
-  lua_util.debugm(N, task, "gpt from: %s", from_content)
-
-  return url_content, from_content
-end
+-- get_meta_llm_content moved to llm_common
 
 local function check_llm_uncached(task, content, sel_part)
   return settings.specific_check(task, content, sel_part)
@@ -640,12 +647,11 @@ local function openai_check(task, content, sel_part)
   lua_util.debugm(N, task, "sending content to gpt: %s", content)
 
   local upstream
-
   local results = {}
 
-  local function gen_reply_closure(model, idx)
+  local function gen_reply_closure(model, i)
     return function(err, code, body)
-      results[idx].checked = true
+      results[i].checked = true
       if err then
         rspamd_logger.errx(task, '%s: request failed: %s', model, err)
         upstream:fail()
@@ -662,15 +668,15 @@ local function openai_check(task, content, sel_part)
 
       local reply, reason, categories = settings.reply_conversion(task, body)
 
-      results[idx].model = model
+      results[i].model = model
 
       if reply then
-        results[idx].success = true
-        results[idx].probability = reply
-        results[idx].reason = reason
+        results[i].success = true
+        results[i].probability = reply
+        results[i].reason = reason
 
         if categories then
-          results[idx].categories = categories
+          results[i].categories = categories
         end
       end
 
@@ -678,52 +684,61 @@ local function openai_check(task, content, sel_part)
     end
   end
 
-  local from_content, url_content = get_meta_llm_content(task)
+  -- Build messages exactly as in the original code if structured table provided
+  local user_messages
+  if type(content) == 'table' then
+    local subject_line = 'Subject: ' .. (content.subject or '')
+    user_messages = {
+      { role = 'user', content = subject_line },
+      { role = 'user', content = content.from or '' },
+      { role = 'user', content = content.url_domains or '' },
+      { role = 'user', content = content.text or '' },
+    }
+  else
+    user_messages = {
+      { role = 'user', content = content }
+    }
+  end
 
-  local body = {
-    model = settings.model,
-    max_tokens = settings.max_tokens,
-    temperature = settings.temperature,
+  local body_base = {
+    stream = false,
     messages = {
       {
         role = 'system',
         content = settings.prompt
       },
-      {
-        role = 'user',
-        content = 'Subject: ' .. task:get_subject() or '',
-      },
-      {
-        role = 'user',
-        content = from_content,
-      },
-      {
-        role = 'user',
-        content = url_content,
-      },
-      {
-        role = 'user',
-        content = content
-      }
+      lua_util.unpack(user_messages)
     }
   }
 
-  -- Conditionally add response_format
-  if settings.include_response_format then
-    body.response_format = { type = "json_object" }
-  end
-
   if type(settings.model) == 'string' then
     settings.model = { settings.model }
   end
 
-  upstream = settings.upstreams:get_upstream_round_robin()
   for idx, model in ipairs(settings.model) do
     results[idx] = {
       success = false,
       checked = false
     }
+    -- Fresh body for each model
+    local body = lua_util.deepcopy(body_base)
+
+    -- Merge model-specific parameters into body
+    local params = settings.model_parameters[model]
+    if params then
+      for k, v in pairs(params) do
+        body[k] = v
+      end
+    end
+
+    -- Conditionally add response_format
+    if settings.include_response_format then
+      body.response_format = { type = "json_object" }
+    end
+
     body.model = model
+
+    upstream = settings.upstreams:get_upstream_round_robin()
     local http_params = {
       url = settings.url,
       mime_type = 'application/json',
@@ -738,12 +753,16 @@ local function openai_check(task, content, sel_part)
       task = task,
       upstream = upstream,
       use_gzip = true,
+      -- staged timeouts
+      connect_timeout = settings.connect_timeout,
+      ssl_timeout = settings.ssl_timeout,
+      write_timeout = settings.write_timeout,
+      read_timeout = settings.read_timeout,
     }
 
     if not rspamd_http.request(http_params) then
       results[idx].checked = true
     end
-
   end
 end
 
@@ -753,9 +772,9 @@ local function ollama_check(task, content, sel_part)
   local upstream
   local results = {}
 
-  local function gen_reply_closure(model, idx)
+  local function gen_reply_closure(model, i)
     return function(err, code, body)
-      results[idx].checked = true
+      results[i].checked = true
       if err then
         rspamd_logger.errx(task, '%s: request failed: %s', model, err)
         upstream:fail()
@@ -770,65 +789,78 @@ local function ollama_check(task, content, sel_part)
         return
       end
 
-      local reply, reason = settings.reply_conversion(task, body)
+      local reply, reason, categories = settings.reply_conversion(task, body)
 
-      results[idx].model = model
+      results[i].model = model
 
       if reply then
-        results[idx].success = true
-        results[idx].probability = reply
-        results[idx].reason = reason
+        results[i].success = true
+        results[i].probability = reply
+        results[i].reason = reason
+        if categories then
+          results[i].categories = categories
+        end
       end
 
       check_consensus_and_insert_results(task, results, sel_part)
     end
   end
 
-  local from_content, url_content = get_meta_llm_content(task)
+  local user_messages
+  if type(content) == 'table' then
+    local subject_line = 'Subject: ' .. (content.subject or '')
+    user_messages = {
+      { role = 'user', content = subject_line },
+      { role = 'user', content = content.from or '' },
+      { role = 'user', content = content.url_domains or '' },
+      { role = 'user', content = content.text or '' },
+    }
+  else
+    user_messages = {
+      { role = 'user', content = content }
+    }
+  end
 
   if type(settings.model) == 'string' then
     settings.model = { settings.model }
   end
 
-  local body = {
+  local body_base = {
     stream = false,
     model = settings.model,
-    max_tokens = settings.max_tokens,
-    temperature = settings.temperature,
+    -- should not in body_base
+    -- max_tokens = settings.max_tokens,
+    -- temperature = settings.temperature,
     messages = {
       {
         role = 'system',
         content = settings.prompt
       },
-      {
-        role = 'user',
-        content = 'Subject: ' .. task:get_subject() or '',
-      },
-      {
-        role = 'user',
-        content = from_content,
-      },
-      {
-        role = 'user',
-        content = url_content,
-      },
-      {
-        role = 'user',
-        content = content
-      }
+      table.unpack(user_messages)
     }
   }
 
-  for i, model in ipairs(settings.model) do
+  for idx, model in ipairs(settings.model) do
+    results[idx] = {
+      success = false,
+      checked = false
+    }
+    -- Fresh body for each model
+    local body = lua_util.deepcopy(body_base)
+
+    -- Merge model-specific parameters into body
+    local params = settings.model_parameters[model]
+    if params then
+      for k, v in pairs(params) do
+        body[k] = v
+      end
+    end
+
     -- Conditionally add response_format
     if settings.include_response_format then
       body.response_format = { type = "json_object" }
     end
 
-    results[i] = {
-      success = false,
-      checked = false
-    }
     body.model = model
 
     upstream = settings.upstreams:get_upstream_round_robin()
@@ -837,15 +869,22 @@ local function ollama_check(task, content, sel_part)
       mime_type = 'application/json',
       timeout = settings.timeout,
       log_obj = task,
-      callback = gen_reply_closure(model, i),
+      callback = gen_reply_closure(model, idx),
       keepalive = true,
       body = ucl.to_format(body, 'json-compact', true),
       task = task,
       upstream = upstream,
       use_gzip = true,
+      -- staged timeouts
+      connect_timeout = settings.connect_timeout,
+      ssl_timeout = settings.ssl_timeout,
+      write_timeout = settings.write_timeout,
+      read_timeout = settings.read_timeout,
     }
 
-    rspamd_http.request(http_params)
+    if not rspamd_http.request(http_params) then
+      results[idx].checked = true
+    end
   end
 end
 
@@ -929,7 +968,6 @@ if opts then
   if not settings.api_key and llm_type.require_passkey then
     rspamd_logger.warnx(rspamd_config, 'no api_key is specified for LLM type %s, disabling module', settings.type)
     lua_util.disable_module(N, "config")
-
     return
   end
 
@@ -948,12 +986,14 @@ if opts then
     type = 'virtual',
     parent = id,
     score = 3.0,
+    group = 'GPT',
   })
   rspamd_config:register_symbol({
     name = 'GPT_HAM',
     type = 'virtual',
     parent = id,
     score = -2.0,
+    group = 'GPT',
   })
 
   if settings.extra_symbols then
@@ -962,7 +1002,8 @@ if opts then
         name = sym,
         type = 'virtual',
         parent = id,
-        score = data.score,
+        score = data.score or 0,
+        group = data.group or 'GPT',
         description = data.description,
       })
       data.name = sym
@@ -977,7 +1018,8 @@ if opts then
           "Output ONLY 3 lines:\n" ..
           "1. Numeric score (0.00-1.00)\n" ..
           "2. One-sentence reason citing whether it is spam, the strongest red flag, or why it is ham\n" ..
-          "3. Primary concern category if found from the list: " .. table.concat(lua_util.keys(categories_map), ', ')
+          "3. Empty line or mention ONLY the primary concern category if found from the list: " .. 
+            table.concat(lua_util.keys(categories_map), ', ')
     else
       settings.prompt = "Analyze this email strictly as a spam detector given the email message, subject, " ..
           "FROM and url domains. Evaluate spam probability (0-1). " ..
